/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package sjc.ui;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

/**
 * GetMthd: search ip in file created by debug.CodeInfo
 * 
 * @author S. Frenz
 * @version 091105 initial version
 */

public class GetMthd {
  public static void main(String[] args) {
    BufferedReader br;
    int ip;
    
    if (args==null || args.length!=2) {
      System.out.println("GetMthd for code info generated by SC");
      System.out.println("Please give a file and a code address as parameter");
      return;
    }
    
    try {
      br=new BufferedReader(new FileReader(args[0]));
    }
    catch (IOException e) {
      System.out.println("Could not open file: "+e.getMessage());
      return;
    }
    try {
      if (args[1].startsWith("0x")) ip=Integer.parseInt(args[1].substring(2), 16);
      else ip=Integer.parseInt(args[1]);
    }
    catch (NumberFormatException e) {
      System.out.println("Invalid code address: "+e.getMessage());
      return;
    }
    
    System.out.println("searching for 0x"+Integer.toHexString(ip)+" in file "+args[0]);
    
    try {
      String line;
      while ((line=br.readLine())!=null) {
        if (line.startsWith("code:")) {
          String[] token=line.split(" ");
          int start=Integer.parseInt(token[1].substring(2), 16), size=Integer.parseInt(token[2]);
          if (start<=ip && ip<start+size) { //found method
            printMethodInfo(br, ip-start);
            br.close();
            return;
          }
        }
      }
      br.close();
      System.out.println("address not found");
    }
    catch (IOException e) {
      System.out.println("I/O-error: "+e.getMessage());
      return;
    }
    catch (NumberFormatException e) {
      System.out.println("invalid number in file");
    }
  }
  
  private static void printMethodInfo(BufferedReader br, int offset) throws IOException,NumberFormatException {
    String line, slhiLine=null;
    
    while ((line=br.readLine())!=null && !line.equals("")) {
      if (line.startsWith("slhi:") || line.startsWith(" ")) {
        String[] token=line.split(" ");
        for (int i=1; i+1<token.length; i+=2)
          if (Integer.parseInt(token[i])<=offset)
            slhiLine=token[i+1];
      }
      else System.out.println(line);
    }
    if (slhiLine!=null) System.out.println("slhi: "+slhiLine);
  }
}
